from django.db import connections
from django.db.backends.base.base import _thread

from arkindex.documents.models import ElementPath
from arkindex.documents.tasks import move_element
from arkindex.project.tests import FixtureTestCase


class TestMoveElement(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.destination = cls.corpus.elements.get(name="Volume 2")
        cls.parent = cls.corpus.elements.get(name="Volume 1")
        cls.source_with_children = cls.corpus.elements.get(name="Volume 1, page 1r")
        cls.source_without_child = cls.corpus.elements.get(name="Volume 1, page 2r")
        ElementPath.objects.filter(path__contains=[cls.source_without_child.id]).delete()

    def test_run_on_source_without_child(self):
        # No child on this page
        self.assertEqual(ElementPath.objects.filter(path__contains=[self.source_without_child.id]).count(), 0)

        source_paths = ElementPath.objects.filter(element_id=self.source_without_child.id)
        self.assertEqual(len(source_paths), 1)
        self.assertEqual(list(source_paths.values("path")), [{"path": [self.parent.id]}])

        with self.assertExactQueries("element_move_without_child.sql", params={
            "source_id": str(self.source_without_child.id),
            "parent_id": str(self.parent.id),
            "destination_id": str(self.destination.id),
            "savepoints": [f"s{_thread.get_ident()}_x{connections['default'].savepoint_state + 1}", f"s{_thread.get_ident()}_x{connections['default'].savepoint_state + 2}"]
        }):
            move_element(self.source_without_child, self.destination)

        self.assertEqual(len(source_paths), 1)
        self.assertEqual(list(source_paths.values("path")), [{"path": [self.destination.id]}])

    def test_run_on_source_with_children(self):
        # 4 children on this page
        children_paths = ElementPath.objects.filter(path__contains=[self.source_with_children.id])
        self.assertEqual(children_paths.count(), 4)
        self.assertEqual(list(children_paths.values("path")), [
            {"path": [self.parent.id, self.source_with_children.id]},
            {"path": [self.parent.id, self.source_with_children.id]},
            {"path": [self.parent.id, self.source_with_children.id]},
            {"path": [self.parent.id, self.source_with_children.id]}
        ])

        source_paths = ElementPath.objects.filter(element_id=self.source_with_children.id)
        self.assertEqual(len(source_paths), 1)
        self.assertEqual(list(source_paths.values("path")), [{"path": [self.parent.id]}])

        with self.assertExactQueries("element_move_with_children.sql", params={
            "source_id": str(self.source_with_children.id),
            "parent_id": str(self.parent.id),
            "destination_id": str(self.destination.id),
            "savepoints": [f"s{_thread.get_ident()}_x{connections['default'].savepoint_state + 1}", f"s{_thread.get_ident()}_x{connections['default'].savepoint_state + 2}"]
        }):
            move_element(self.source_with_children, self.destination)

        self.assertEqual(len(source_paths), 1)
        self.assertEqual(list(source_paths.values("path")), [{"path": [self.destination.id]}])
        # Assert children were also moved
        self.assertEqual(list(children_paths.values("path")), [
            {"path": [self.destination.id, self.source_with_children.id]},
            {"path": [self.destination.id, self.source_with_children.id]},
            {"path": [self.destination.id, self.source_with_children.id]},
            {"path": [self.destination.id, self.source_with_children.id]}
        ])
